/************************************************************************
*		     JPassManage version 3.0									*
*   Management of login and password for sites or software				*
*		       Software for Palm Os										*
*                Copyright (C) 2002  Arnaud FARINE 						*
*		    <arnaud.farine@wanadoo.fr>									*
* This program is free software; you can redistribute it and/or			*
* modify it under the terms of the GNU General Public License			*
* as published by the Free Software Foundation; either version 2		*
* of the License, or (at your option) any later version.				*
*																		*
* This program is distributed in the hope that it will be useful,		*
* but WITHOUT ANY WARRANTY; without even the implied warranty of		*
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			*
* GNU General Public License for more details.							*
*																		*
* You should have received a copy of the GNU General Public License		*
* along with this program; if not, write to 							*
*                    Free Software Foundation, Inc.,					*
*		      59 Temple Place - Suite 330,								*
*		      Boston, MA  02111-1307, USA.								*
************************************************************************* 

/**
* JPassManage
* @author Arnaud FARINE
* @version 2.0
*/

import waba.io.*;
import waba.util.Vector;
import waba.ui.*;
import waba.sys.Vm;
import waba.sys.Convert;
/**
* Cette classe permet de grer la base de donnes
*/
public class AccesDaoPDB  {
	private Catalog db;
	
	/**
	* Le constructeur
	*/
	public AccesDaoPDB(){
		db = new Catalog("JPM22.AFR0.JPM2",Catalog.READ_WRITE);
		if (!db.isOpen()){
			db=new Catalog("JPM22.AFR0.JPM2",Catalog.CREATE);
			db.close();
			db = new Catalog("JPM22.AFR0.JPM2",Catalog.READ_WRITE);
			initPdb();
		}

					
	}
	
	private void initPdb(){
			DataStream ds = new DataStream(db);
			int i = db.addRecord(50);
			ds.writeString("");
			db.addRecord(50,2);
			ds.writeString("");
			db.addRecord(50,3);
			ds.writeString("");
			db.addRecord(50,4);
			ds.writeString("");
			db.addRecord(50,5);
			ds.writeString("");
	}
	/**
	* Mthode d'enregistrement d'un triplet d'information EnregIdPwd(site, identifiant, password)
	* @param eip Le triplet site, login, pass
	* @return 0 si l'opration a reussi, -1 en cas d'erreur
	*/
	public int saveEnreg(EnregIdPwd eip){
		int j=0;
    	if (!db.isOpen())return -1;
    	DataStream ds = new DataStream(db);
    	int dernierId=-1;
    	dernierId=plusGrandId(ds)+1;
    	j = db.addRecord(6);
    	if (j==-1) {return -1;}
    	ds.writeInt(dernierId); // ID
		j = db.addRecord(50);
		ds.writeString(eip.getSite()); //Le site
		j = db.addRecord(50);
		ds.writeString(eip.getLog());//Le login
		j = db.addRecord(50);
		ds.writeString(eip.getPwd());//Le password
		j = db.addRecord(100);
		ds.writeString(eip.getInfoSupp());//L'information supplmentaire
		return 0;
	}
	
	
	
	/**
	* Accde en lecture  tous les enregistrements
	* @return Le vecteur contenant des objets EnregIdPwd et correpsondant aux informations contenus
	* 		dans la base
	*/
	public Vector readAllEnreg(){
		Vector ensenreg=new Vector();	
	    if (!db.isOpen()) return ensenreg;
		int r=db.getRecordCount();
		DataStream ds = new DataStream(db);
		
		int tmpcount=1;
		EnregIdPwd eip=null;
		for (int i=5;i<r;i++){
			db.setRecordPos(i);
			switch (tmpcount){
				case 1:
					eip = new EnregIdPwd();
					eip.setId(ds.readInt());
					tmpcount++;
					break;
				case 2:
					eip.setSite(ds.readString());
					tmpcount++;
					break;
				case 3:
					eip.setLog(ds.readString());
					tmpcount++;
					break;
				case 4:
					eip.setPwd(ds.readString());
					tmpcount++;
					break;
				case 5:
					eip.setInfoSupp(ds.readString());
					ensenreg.add(eip);
					tmpcount=1;
					break;
			}
		}
		ds.close();
		return ensenreg;
	}
	

	/**
	* Supprime un enregistrement de la base (1 enregistrement EnregIdPwd est gal  3 enregistrements dans la base)
	* @param indiceenreg Le numro d'enregistrement de l'information qu'il faut rechercher pour effacer les donnes le concernant
	* @return 0 en cas de suppresion reussie ou -1 en cas d'chec
	*/
	public int suppEnreg(int indiceenreg){
		if(!db.isOpen()) return -1;
	    DataStream ds = new DataStream(db);
	    int indiceFind = trouverEnreg(indiceenreg,ds);
	    if (indiceFind==-1) return -1;
		for (int ind=0;ind<5;ind++){		
			db.setRecordPos(indiceFind);
			db.deleteRecord();
		}
		return 0;
	}
	
	/**
	* Recherche un enregistrement dans le catalogue
	* @param delenreg Le numero Id de l'enregistrement
	*		 dst Le DataStream li au catalogue actuellement ouvert
	* @return 1 en cas de russite, 0 en cas d'chec
	*/
	private int trouverEnreg(int delenreg, DataStream dst){
		DataStream myds = dst;
		int rec=db.getRecordCount();
		int tmpcount=1;
		for (int i=5;i<rec;i++){
			db.setRecordPos(i);
			switch (tmpcount){
				case 1:
					int mydel=(int)myds.readInt();
					if (delenreg==mydel){
						return i;
					}
					tmpcount++;
					break;
				case 2:
					tmpcount++;
					break;
				case 3:
					tmpcount++;
					break;
				case 4:
					tmpcount++;
					break;
				case 5:
					tmpcount=1;
					break;
			}
	 	}
	 	return -1;
	 }
	 
	 /**
	 * Retourne le plus grand numero d'enregsitrement trouv dans la base
	 * @param d Le dataStream utilis sur la base de donnes
	 * @return Le numro de l'enregistrement (et non la position)
	 */
	 private int plusGrandId(DataStream d){
	 	int pgi=-1;
	 	int tmp=0;
	 	int tailleCat=db.getRecordCount();
	 	if (tailleCat==0) return 0;
	 	int pos=1;
	 	for (int i=0;i<tailleCat;i++){
	 		switch (pos){
	 			case 1:
	 				db.setRecordPos(i);
	 				tmp=d.readInt();
	 				if (tmp>pgi) pgi=tmp;
	 				pos++;
	 				break;
	 			case 2:
	 				pos++;
	 				break;
	 			case 3:
	 				pos++;
	 				break;
	 			case 4:
	 				pos++;
	 				break;
	 			case 5:
	 				pos=1;
	 				break;
	 		}
	 	}
	 	return pgi;
	 }
	 /**
	 * Ferme l'accs  la base de donnes
	 */
	 public void dbClose(){
	 	db.close();
	 }
	 /**
	 * Exporte les donnes contenues de l'application dans un fiche du bloc-notes
	 * @param nomfich - Le nom d'en-tete du mmo
	 * @return Le rsultat de l'enregistrement, 0 si ok un nombre ngatif dans le cas contraire
	 */ 
	 public int expData(String nomfich){
		if (!db.isOpen()) {
			return -1;
		}
		Vector vectlu = readAllEnreg();
		int longVect=vectlu.getCount();
		if (longVect==0){
			db.close();
			return -2;
		}
		db.close();
		String strb=nomfich + "\n";
		int ligneenreg=0;
		for (int i=5;i<longVect;i++){
			EnregIdPwd eip = (EnregIdPwd)vectlu.elementAt(i);
			for (int j=0;j<4;j++){
				switch (ligneenreg){
					case 0:
						strb = strb + "Site: " + eip.getSite() + "\n";
						ligneenreg++;
						break;
					case 1:
						strb= strb + "Login: " + eip.getLog() + "\n";	
						ligneenreg++;
						break;
					case 2:
						strb= strb + "Mot de passe: " + eip.getPwd() + "\n";	
						ligneenreg=0;
					case 3:
						strb= strb + "Remarque: " + eip.getInfoSupp() + "\n";	
						ligneenreg=0;
				}
			}
		}
		Catalog dbmemo = new Catalog("MemoDB.memo.DATA",Catalog.READ_WRITE);
		if (!dbmemo.isOpen()){return -3;}
		DataStream dsmemo = new DataStream(dbmemo);
		int j=0;
		j=dbmemo.addRecord(16*strb.length());
		if (j==-1) return -4;
		dsmemo.writeCString(strb);
		dsmemo.close();
		dbmemo.close();
	 	return 0;	
	 }
	 
	 /**
	 * Cette mthode permet d'importer des donnes au sein de l'application JPassManage
	 *  partir d'une note contenu dans le Catalog Bloc-notes. La note devra contenir
	 * une premire ligne contenant le nom de la note. Chaque triplet sera compos comme suit
	 * afin de pouvoir tre traiter:
	 * Site:<espace>monsite
	 * Login:<espace>monlogin
	 * Mot de passe:<espace>monmotdepsse
	 * Rem :<espace>ma remarque
	 * Chaque fin de ligne  contenant un retour charriot
	 * @param le nom de la note contenant les informations
	 * @return Un code retour : 0 si tout s'est bien pass, un nombre ngatif dans le cas contraire
	 */
	 public int impData(String nomfich){
	 	Catalog dbmemo = new Catalog("MemoDB.memo.DATA",Catalog.READ_ONLY);
		if (!dbmemo.isOpen())return -1;
		int nbenreg = dbmemo.getRecordCount();
		if (nbenreg==0) return -2;
		DataStream dsmemo = new DataStream(dbmemo);
		String note="";
		int fichiertrouve=0;
		for (int i=0;i<nbenreg;i++){
			dbmemo.setRecordPos(i);
			note = dsmemo.readCString();
			if (note.startsWith(nomfich)) {
				fichiertrouve=1;
				break;
			} 
		}
		dsmemo.close();
		dbmemo.close();
		if (fichiertrouve==1){
			note = note.substring(nomfich.length(),note.length());
			Vector v = parserImpData(note);	
			int sizev = v.getCount();
			if (sizev==0) return -3;
			else {
				for (int d=0;d<sizev;d++){
					EnregIdPwd eip = (EnregIdPwd)v.elementAt(d);
					if (eip.objString().equals("")) return -5;
					if (saveEnreg(eip)==-1){
						return -6;	
					}
				}	
			}
		}
		else return -7;
		
		return 0;
	 }
	 
	 private Vector parserImpData(String s){
	 	String[] lesData = Convert.tokenizeString(s,'\n');
	 	int longdonnees = lesData.length;
	 	int count=0;
	 	Vector dataToRecord= new Vector();
	 	String site="";
	 	String identifiant="";
	 	String password="";
	 	String remarque = "";
	 	for (int k=1;k<longdonnees;k++){
	 		switch (count){
	 			case 0:
	 				String[] parsesite = Convert.tokenizeString(lesData[k],':');
	 				int tailleparsesite = parsesite.length;
	 				for (int a=1;a<tailleparsesite;a++){
	 					site = site + parsesite[a];
	 				}
	 				count=count+1;
	 				break;
	 			case 1:
	 				String[] parseid = Convert.tokenizeString(lesData[k],':');
	 				int tailleparseid = parseid.length;
	 				for (int b=1;b<tailleparseid;b++){
	 					identifiant = identifiant + parseid[b];
	 				}
	 				count=count+1;
	 				break;
	 			case 2:
	 				String[] parsepwd = Convert.tokenizeString(lesData[k],':');
	 				int tailleparsepwd = parsepwd.length;
	 				for (int c=1;c<tailleparsepwd;c++){
	 					password = password + parsepwd[c];
	 				}
	 				count=count+1;
	 				break;
	 			case 3:
	 				String[] parserem = Convert.tokenizeString(lesData[k],':');
	 				int tailleparserem = parserem.length;
	 				for (int d=1;d<tailleparserem;d++){
	 					remarque = remarque + parserem[d];
	 				}
	 				
	 				EnregIdPwd eip = new EnregIdPwd();
			 		eip.setSite(site);
			 		eip.setLog(identifiant);
					eip.setPwd(password);
					eip.setInfoSupp(remarque);
			 		dataToRecord.add(eip);
	 				site="";
	 				identifiant="";
	 				password="";
	 				remarque="";
	 				count=0;
	 				break;	
	 		}
	 		
	 	}
	 	return dataToRecord;
	 }
	 
	 public User getUser(){
	 	if (!db.isOpen()) return null;
	 	DataStream ds = new DataStream(db);
	 	User u = new User();
	 	String tmp="";
	 	for (int i=0;i<5;i++){
	 		boolean j=db.setRecordPos(i);
	 		Vm.debug("record lu : " + i + " :" + j);
	 		tmp=ds.readString();	
	 		switch(i){
	 			case 0:
	 				u.setParam1(tmp);
	 				break;
	 			case 1:
	 				u.setParam2(tmp);
	 				break;
	 			case 2 :
	 				u.setParam3(tmp);
	 				break;
	 			case 3 :
	 				u.setParam4(tmp);
	 				break;
	 			case 4 :
	 				u.setParam5(tmp);
	 				break;
	 		}
	 	}
	 	db.close();
	 	return u;
	 }
	 
	 public int setUser(User u){
	 	if (!db.isOpen()) return -1;
	 	DataStream ds = new DataStream(db);
	 	int j=0;
	 	for (int i=0;i<5;i++){
	 		j = db.addRecord(50,i);
	 		switch (i){
	 			case 0:	
	 				ds.writeString(u.getParam1());
	 			case 1:
	 				ds.writeString(u.getParam2());
	 			case 2 :
	 				ds.writeString(u.getParam3());	
	 			case 3 :
	 				ds.writeString(u.getParam4());	
	 			case 4 :
	 				ds.writeString(u.getParam5());	
	 		}
	 	}
	 	db.close();
	 	return 0;
	 }
}
